import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * SyneEdit
 *
 * @author Kohan "SyneRyder" Ikin
 * @version 1.1
 */

//
//   I realise this code isn't exactly the most well written code out there,
//   so if you have any suggestions on how I can make the code more
//   readable please email me at syneryder@namesuppressed.com and I'll
//   make the changes in the next release.
//
//   The latest version will be always be available from
//   http://www.namesuppressed.com/syneryder/
//   

public class SyneEdit extends Frame implements ActionListener, WindowListener {

    // The components in our GUI
    MenuBar     menuBar;
    Menu        fileMenu;
    Menu        editMenu;
    MenuItem    save;
    TextArea    text;
    Panel       buttonPanel;

    String currentFileName;
    String clipboard;
    String undo;

    public SyneEdit() {
        super("SyneEdit");
        this.addWindowListener(this);
        currentFileName=null;

        // Construct the components
        menuBar = new MenuBar();
        setMenuBar(menuBar);
        fileMenu = new Menu("File");
        editMenu = new Menu("Edit");
        menuBar.add(fileMenu);
        menuBar.add(editMenu);

        fileMenu.add(new MenuItem("New"));
        fileMenu.add(new MenuItem("Open..."));
        save = new MenuItem("Save");
        save.setEnabled(false);
        fileMenu.add(save);
        fileMenu.add(new MenuItem("Save As..."));
        fileMenu.addSeparator();
        fileMenu.add(new MenuItem("Exit"));
        fileMenu.addActionListener(this);

        editMenu.add(new MenuItem("Undo"));
        editMenu.addSeparator();
        editMenu.add(new MenuItem("Cut"));
        editMenu.add(new MenuItem("Copy"));
        editMenu.add(new MenuItem("Paste"));
        editMenu.addSeparator();
        editMenu.add(new MenuItem("Select All"));
        editMenu.addActionListener(this);

        text = new TextArea("", 20, 80, TextArea.SCROLLBARS_VERTICAL_ONLY);
        text.setEditable(true);
        text.setFont(new Font("Courier", Font.PLAIN, 12));

        buttonPanel = new Panel();

        buttonPanel.add(new Label("SyneEdit v1.1"));
        add(text, "Center");
        add(buttonPanel, "South");

        pack();
        show();
    }


    /**
     * read a file and copy its contents into the text TextArea object
     * @param String fileName, the full path to the file to read
     */

    private void readFile(String fileName) {
        String line="not null";
        try {
            // Construct a BufferedReader
            BufferedReader br = new BufferedReader(new FileReader(fileName));
            line=br.readLine();
            text.setText(line + "\n");
            while(line!=null) {
                line=br.readLine();
                if (line!=null) text.append(line + "\n");
                br.close ();
            }
        } catch(IOException ioe) {
            System.out.println("IO error " + ioe);
        }
    }


    /**
     * write the text in the TextArea object to a file
     * @param String fileName, the full path to the file to write
     */

    private void saveFile(String fileName) {
        String line="not null";
        try {
            BufferedWriter br = new BufferedWriter(new FileWriter(fileName));
            br.write(text.getText());
            br.close();
        } catch (IOException ioe) {
            System.out.println("IO error " + ioe);
        }
    }


    /**
     * process all ActionEvents and carry out appropriate operations
     * @param ActionEvent e, the event to be processed
     */

    public void actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();
        if(command.equals("New")) {
            text.setText("");
            save.setEnabled(false);
            currentFileName = null;
        }
        else if(command.equals("Open...")) {
            FileDialog fd = new FileDialog(this);
            fd.show();
            if ((fd.getDirectory() != null) && (fd.getFile() != null)) {
                currentFileName = fd.getDirectory() + fd.getFile();
                save.setEnabled(true);
                readFile(currentFileName);
            }
        }
        else if(command.equals("Save")) {
            if (save.isEnabled()) {
                if (currentFileName == null) {
                    FileDialog fd = new FileDialog(this);
                    fd.show();
                    currentFileName = fd.getDirectory() + fd.getFile();
                }
                saveFile(currentFileName);
            }
        }
        else if(command.equals("Save As...")) {
            FileDialog fd = new FileDialog(this);
            fd.show();
            if ((fd.getDirectory() != null) && (fd.getFile() != null)) {
                currentFileName = fd.getDirectory() + fd.getFile();
                save.setEnabled(true);
                saveFile(currentFileName);
            }
        }
        else if(command.equals("Exit")) {
            dispose();
            System.exit(0);
        }
    	else if(command.equals("Undo"))
            text.setText(undo);
    	else if(command.equals("Cut")) {
            undo=text.getText();
            clipboard=text.getSelectedText();
            text.replaceRange("", text.getSelectionStart(), text.getSelectionEnd());
    	}
    	else if(command.equals("Copy"))
            clipboard=text.getSelectedText();
    	else if(command.equals("Paste")) {
            undo=text.getText();
            if (text.getSelectedText() == "")
            	text.insert(clipboard, text.getCaretPosition());
            else text.replaceRange(clipboard, text.getSelectionStart(), text.getSelectionEnd());
    	}
    	else if(command.equals("Select All"))
            text.selectAll();
    }

    /**
     * process all ActionEvents and carry out appropriate operations
     * @param WindowEvent e, the event to be processed
     */
    public void windowClosing(WindowEvent e) {
        this.dispose();
        System.exit(0);
    }

    // Other unused methods from WindowListener
    public void windowOpened(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowActivated(WindowEvent e) {}
    public void windowDeactivated(WindowEvent e) {}

    public static void main(String[] args) {
        SyneEdit f = new SyneEdit();
    }
}